{% extends "labadmin/base.html" %}
{% load i18n static %}

{% block branding %}
<h1 id="site-name">{% trans "Registration" %}</h1>
{% endblock %}

{% block content %}
<div id="content-main">
   <h2>{% trans "Thanks for completing the registration" %}</h2>
   <p>{% trans "Now you can" %} <a href="{% url 'auth_login' %}?next={% url 'labadmin-user-profile' %}">{% trans "Log in" %}</a></p>
</div>
{% endblock %}
