# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-10-11 12:31
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('labadmin', '0002_auto_20160927_1249'),
    ]

    operations = [
        migrations.AddField(
            model_name='card',
            name='credits',
            field=models.IntegerField(default=0),
        ),
    ]
