{% extends "admin/base_site.html" %}
{% load i18n static %}

{% block extrastyle %}{{ block.super }}<link rel="stylesheet" type="text/css" href="{% static "admin/css/login.css" %}" />
{{ form.media }}
<style type="text/css">
#userprofile_form .form-row > input, #userprofile_form .form-row > textarea, #registration_form .form-row > input {
    clear: both;
    padding: 8px;
    width: 100%;
    box-sizing: border-box;
}
</style>
{% endblock %}

{% block title %}{{ site_title|default:"LabAdmin" }}{% endblock %}

{% block bodyclass %}{{ block.super }}{% endblock %}

{% block usertools %}
{% if request.user.is_authenticated %}
    <div id="user-tools">
        {% block welcome-msg %}
            {% trans 'Welcome,' %}
            <strong>{% firstof user.get_short_name user.get_username %}</strong>.
        {% endblock %}
        {% block userlinks %}
            {% if user.has_usable_password %}
            <a href="{% url 'labadmin-password-change' %}">{% trans 'Change password' %}</a> /
            {% endif %}
            <a href="{% url 'labadmin-logout' %}">{% trans 'Log out' %}</a>
        {% endblock %}
    </div>
{% endif %}
{% endblock %}

{% block nav-global %}{% endblock %}

{% block content_title %}{% endblock %}

{% block breadcrumbs %}{% endblock %}

{% block content %}

{% if form.errors and not form.non_field_errors %}
<p class="errornote">
{% if form.errors.items|length == 1 %}{% trans "Please correct the error below." %}{% else %}{% trans "Please correct the errors below." %}{% endif %}
</p>
{% endif %}

{% if form.non_field_errors %}
{% for error in form.non_field_errors %}
<p class="errornote">
    {{ error }}
</p>
{% endfor %}
{% endif %}

{% endblock %}
