# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2017-03-02 10:31
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('labadmin', '0009_logerror_device'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='logaccess',
            name='users',
        ),
    ]
