# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2017-01-18 19:12
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('labadmin', '0006_device_token'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='role',
            name='role_kind',
        ),
    ]
