# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2017-03-02 12:09
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('labadmin', '0010_remove_logaccess_users'),
    ]

    operations = [
        migrations.CreateModel(
            name='Sketch',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100)),
                ('code', models.TextField()),
                ('file_format', models.CharField(default='.txt', max_length=10)),
            ],
            options={
                'verbose_name_plural': 'Sketches',
            },
        ),
    ]
