{% extends "labadmin/base.html" %}
{% load i18n static %}

{% block branding %}{% endblock %}

{% block content %}

<h1>{% trans "Welcome to LabAdmin" %}</h1>

{% if request.user.is_authenticated %}
<p><a href="{% url "labadmin-user-profile" %}">{% trans "User profile" %}</a></p>
<p><a href="{% url "labadmin-logout" %}">{% trans "Log out" %}</a></p>
{% else %}
<p>{% trans "If you need an account please" %} <a href="{% url "registration_register" %}">{% trans "Register" %}</a></p>
<p>{% trans "If you already have an account please" %} <a href="{% url "login" %}">{% trans "Log in" %}</a></p>
{% endif %}

</div>
{% endblock %}
