# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-10-12 12:23
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('labadmin', '0004_logcredits'),
    ]

    operations = [
        migrations.AlterField(
            model_name='userprofile',
            name='card',
            field=models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='labadmin.Card'),
        ),
    ]
