# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-05-18 21:58
from __future__ import unicode_literals

from django.db import migrations
import versatileimagefield.fields


class Migration(migrations.Migration):

    dependencies = [
        ('labadmin', '0011_sketch'),
    ]

    operations = [
        migrations.AlterField(
            model_name='userprofile',
            name='picture',
            field=versatileimagefield.fields.VersatileImageField(blank=True, null=True, upload_to='labadmin/users/pictures'),
        ),
    ]
